@@ -23,7 +23,6 @@
 """Git object store interfaces and implementation."""
 
 from io import BytesIO
-import errno
 import os
 import stat
 import sys
@@ -48,6 +47,7 @@ from dulwich.objects import (
     hex_to_filename,
     S_ISGITLINK,
     object_class,
+    valid_hexsha,
     )
 from dulwich.pack import (
     Pack,
@@ -82,8 +82,9 @@ class BaseObjectStore(object):
     def iter_shas(self, shas):
         """Iterate over the objects for the specified shas.
 
-        :param shas: Iterable object with SHAs
-        :return: Object iterator
+        Args:
+          shas: Iterable object with SHAs
+        Returns: Object iterator
         """
         return ObjectStoreIterator(self, shas)
 
@@ -110,8 +111,9 @@ class BaseObjectStore(object):
     def get_raw(self, name):
         """Obtain the raw text for an object.
 
-        :param name: sha for the object.
-        :return: tuple with numeric type and object contents.
+        Args:
+          name: sha for the object.
+        Returns: tuple with numeric type and object contents.
         """
         raise NotImplementedError(self.get_raw)
 
@@ -133,22 +135,26 @@ class BaseObjectStore(object):
     def add_objects(self, objects, progress=None):
         """Add a set of objects to this object store.
 
-        :param objects: Iterable over a list of (object, path) tuples
+        Args:
+          objects: Iterable over a list of (object, path) tuples
         """
         raise NotImplementedError(self.add_objects)
 
     def add_pack_data(self, count, pack_data, progress=None):
         """Add pack data to this object store.
 
-        :param num_items: Number of items to add
-        :param pack_data: Iterator over pack data tuples
+        Args:
+          num_items: Number of items to add
+          pack_data: Iterator over pack data tuples
         """
         if count == 0:
             # Don't bother writing an empty pack file
             return
         f, commit, abort = self.add_pack()
         try:
-            write_pack_data(f, count, pack_data, progress)
+            write_pack_data(
+                f, count, pack_data, progress,
+                compression_level=self.pack_compression_level)
         except BaseException:
             abort()
             raise
@@ -156,22 +162,25 @@ class BaseObjectStore(object):
             return commit()
 
     def tree_changes(self, source, target, want_unchanged=False,
-                     include_trees=False, change_type_same=False):
+                     include_trees=False, change_type_same=False,
+                     rename_detector=None):
         """Find the differences between the contents of two trees
 
-        :param source: SHA1 of the source tree
-        :param target: SHA1 of the target tree
-        :param want_unchanged: Whether unchanged files should be reported
-        :param include_trees: Whether to include trees
-        :param change_type_same: Whether to report files changing
+        Args:
+          source: SHA1 of the source tree
+          target: SHA1 of the target tree
+          want_unchanged: Whether unchanged files should be reported
+          include_trees: Whether to include trees
+          change_type_same: Whether to report files changing
             type in the same entry.
-        :return: Iterator over tuples with
+        Returns: Iterator over tuples with
             (oldpath, newpath), (oldmode, newmode), (oldsha, newsha)
         """
         for change in tree_changes(self, source, target,
                                    want_unchanged=want_unchanged,
                                    include_trees=include_trees,
-                                   change_type_same=change_type_same):
+                                   change_type_same=change_type_same,
+                                   rename_detector=rename_detector):
             yield ((change.old.path, change.new.path),
                    (change.old.mode, change.new.mode),
                    (change.old.sha, change.new.sha))
@@ -181,9 +190,10 @@ class BaseObjectStore(object):
 
         Iteration is depth-first pre-order, as in e.g. os.walk.
 
-        :param tree_id: SHA1 of the tree.
-        :param include_trees: If True, include tree objects in the iteration.
-        :return: Iterator over TreeEntry namedtuples for all the objects in a
+        Args:
+          tree_id: SHA1 of the tree.
+          include_trees: If True, include tree objects in the iteration.
+        Returns: Iterator over TreeEntry namedtuples for all the objects in a
             tree.
         """
         for entry, _ in walk_trees(self, tree_id, None):
@@ -191,31 +201,34 @@ class BaseObjectStore(object):
                  not stat.S_ISDIR(entry.mode)) or include_trees):
                 yield entry
 
-    def find_missing_objects(self, haves, wants, progress=None,
+    def find_missing_objects(self, haves, wants, shallow=None, progress=None,
                              get_tagged=None,
                              get_parents=lambda commit: commit.parents,
                              depth=None):
         """Find the missing objects required for a set of revisions.
 
-        :param haves: Iterable over SHAs already in common.
-        :param wants: Iterable over SHAs of objects to fetch.
-        :param progress: Simple progress function that will be called with
+        Args:
+          haves: Iterable over SHAs already in common.
+          wants: Iterable over SHAs of objects to fetch.
+          shallow: Set of shallow commit SHA1s to skip
+          progress: Simple progress function that will be called with
             updated progress strings.
-        :param get_tagged: Function that returns a dict of pointed-to sha ->
+          get_tagged: Function that returns a dict of pointed-to sha ->
             tag sha for including tags.
-        :param get_parents: Optional function for getting the parents of a
+          get_parents: Optional function for getting the parents of a
             commit.
-        :return: Iterator over (sha, path) pairs.
+        Returns: Iterator over (sha, path) pairs.
         """
-        finder = MissingObjectFinder(self, haves, wants, progress, get_tagged,
-                                     get_parents=get_parents)
+        finder = MissingObjectFinder(self, haves, wants, shallow, progress,
+                                     get_tagged, get_parents=get_parents)
         return iter(finder.next, None)
 
     def find_common_revisions(self, graphwalker):
         """Find which revisions this store has in common using graphwalker.
 
-        :param graphwalker: A graphwalker object.
-        :return: List of SHAs that are in common
+        Args:
+          graphwalker: A graphwalker object.
+        Returns: List of SHAs that are in common
         """
         haves = []
         sha = next(graphwalker)
@@ -226,32 +239,39 @@ class BaseObjectStore(object):
             sha = next(graphwalker)
         return haves
 
-    def generate_pack_contents(self, have, want, progress=None):
+    def generate_pack_contents(self, have, want, shallow=None, progress=None):
         """Iterate over the contents of a pack file.
 
-        :param have: List of SHA1s of objects that should not be sent
-        :param want: List of SHA1s of objects that should be sent
-        :param progress: Optional progress reporting method
+        Args:
+          have: List of SHA1s of objects that should not be sent
+          want: List of SHA1s of objects that should be sent
+          shallow: Set of shallow commit SHA1s to skip
+          progress: Optional progress reporting method
         """
-        return self.iter_shas(self.find_missing_objects(have, want, progress))
+        missing = self.find_missing_objects(have, want, shallow, progress)
+        return self.iter_shas(missing)
 
-    def generate_pack_data(self, have, want, progress=None, ofs_delta=True):
+    def generate_pack_data(self, have, want, shallow=None, progress=None,
+                           ofs_delta=True):
         """Generate pack data objects for a set of wants/haves.
 
-        :param have: List of SHA1s of objects that should not be sent
-        :param want: List of SHA1s of objects that should be sent
-        :param ofs_delta: Whether OFS deltas can be included
-        :param progress: Optional progress reporting method
+        Args:
+          have: List of SHA1s of objects that should not be sent
+          want: List of SHA1s of objects that should be sent
+          shallow: Set of shallow commit SHA1s to skip
+          ofs_delta: Whether OFS deltas can be included
+          progress: Optional progress reporting method
         """
         # TODO(jelmer): More efficient implementation
         return pack_objects_to_data(
-            self.generate_pack_contents(have, want, progress))
+            self.generate_pack_contents(have, want, shallow, progress))
 
     def peel_sha(self, sha):
         """Peel all tags from a SHA.
 
-        :param sha: The object SHA to peel.
-        :return: The fully-peeled SHA1 of a tag object, after peeling all
+        Args:
+          sha: The object SHA to peel.
+        Returns: The fully-peeled SHA1 of a tag object, after peeling all
             intermediate tags; if the original ref does not point to a tag,
             this will equal the original SHA1.
         """
@@ -262,16 +282,17 @@ class BaseObjectStore(object):
             obj = self[sha]
         return obj
 
-    def _collect_ancestors(self, heads, common=set(),
+    def _collect_ancestors(self, heads, common=set(), shallow=set(),
                            get_parents=lambda commit: commit.parents):
         """Collect all ancestors of heads up to (excluding) those in common.
 
-        :param heads: commits to start from
-        :param common: commits to end at, or empty set to walk repository
+        Args:
+          heads: commits to start from
+          common: commits to end at, or empty set to walk repository
             completely
-        :param get_parents: Optional function for getting the parents of a
+          get_parents: Optional function for getting the parents of a
             commit.
-        :return: a tuple (A, B) where A - all commits reachable
+        Returns: a tuple (A, B) where A - all commits reachable
             from heads but not present in common, B - common (shared) elements
             that are directly reachable from heads
         """
@@ -285,6 +306,8 @@ class BaseObjectStore(object):
                 bases.add(e)
             elif e not in commits:
                 commits.add(e)
+                if e in shallow:
+                    continue
                 cmt = self[e]
                 queue.extend(get_parents(cmt))
         return (commits, bases)
@@ -296,8 +319,9 @@ class BaseObjectStore(object):
 
 class PackBasedObjectStore(BaseObjectStore):
 
-    def __init__(self):
+    def __init__(self, pack_compression_level=-1):
         self._pack_cache = {}
+        self.pack_compression_level = pack_compression_level
 
     @property
     def alternates(self):
@@ -382,7 +406,7 @@ class PackBasedObjectStore(BaseObjectStore):
     def pack_loose_objects(self):
         """Pack loose objects.
 
-        :return: Number of objects packed
+        Returns: Number of objects packed
         """
         objects = set()
         for sha in self._iter_loose_objects():
@@ -444,8 +468,9 @@ class PackBasedObjectStore(BaseObjectStore):
     def get_raw(self, name):
         """Obtain the raw fulltext for an object.
 
-        :param name: sha for the object.
-        :return: tuple with numeric type and object contents.
+        Args:
+          name: sha for the object.
+        Returns: tuple with numeric type and object contents.
         """
         if name == ZERO_SHA:
             raise KeyError(name)
@@ -456,7 +481,7 @@ class PackBasedObjectStore(BaseObjectStore):
             sha = name
             hexsha = None
         else:
-            raise AssertionError("Invalid object name %r" % name)
+            raise AssertionError("Invalid object name %r" % (name, ))
         for pack in self._iter_cached_packs():
             try:
                 return pack.get_raw(sha)
@@ -484,9 +509,10 @@ class PackBasedObjectStore(BaseObjectStore):
     def add_objects(self, objects, progress=None):
         """Add a set of objects to this object store.
 
-        :param objects: Iterable over (object, path) tuples, should support
+        Args:
+          objects: Iterable over (object, path) tuples, should support
             __len__.
-        :return: Pack object of the objects written.
+        Returns: Pack object of the objects written.
         """
         return self.add_pack_data(
                 *pack_objects_to_data(objects),
@@ -496,19 +522,45 @@ class PackBasedObjectStore(BaseObjectStore):
 class DiskObjectStore(PackBasedObjectStore):
     """Git-style object store that exists on disk."""
 
-    def __init__(self, path):
+    def __init__(self, path, loose_compression_level=-1,
+                 pack_compression_level=-1):
         """Open an object store.
 
-        :param path: Path of the object store.
+        Args:
+          path: Path of the object store.
+          loose_compression_level: zlib compression level for loose objects
+          pack_compression_level: zlib compression level for pack objects
         """
-        super(DiskObjectStore, self).__init__()
+        super(DiskObjectStore, self).__init__(
+            pack_compression_level=pack_compression_level)
         self.path = path
         self.pack_dir = os.path.join(self.path, PACKDIR)
         self._alternates = None
+        self.loose_compression_level = loose_compression_level
+        self.pack_compression_level = pack_compression_level
 
     def __repr__(self):
         return "<%s(%r)>" % (self.__class__.__name__, self.path)
 
+    @classmethod
+    def from_config(cls, path, config):
+        try:
+            default_compression_level = int(config.get(
+                (b'core', ), b'compression').decode())
+        except KeyError:
+            default_compression_level = -1
+        try:
+            loose_compression_level = int(config.get(
+                (b'core', ), b'looseCompression').decode())
+        except KeyError:
+            loose_compression_level = default_compression_level
+        try:
+            pack_compression_level = int(config.get(
+                (b'core', ), 'packCompression').decode())
+        except KeyError:
+            pack_compression_level = default_compression_level
+        return cls(path, loose_compression_level, pack_compression_level)
+
     @property
     def alternates(self):
         if self._alternates is not None:
@@ -521,40 +573,35 @@ class DiskObjectStore(PackBasedObjectStore):
     def _read_alternate_paths(self):
         try:
             f = GitFile(os.path.join(self.path, INFODIR, "alternates"), 'rb')
-        except (OSError, IOError) as e:
-            if e.errno == errno.ENOENT:
-                return
-            raise
+        except FileNotFoundError:
+            return
         with f:
             for line in f.readlines():
                 line = line.rstrip(b"\n")
                 if line[0] == b"#":
                     continue
                 if os.path.isabs(line):
-                    yield line.decode(sys.getfilesystemencoding())
+                    yield os.fsdecode(line)
                 else:
-                    yield os.path.join(self.path, line).decode(
-                        sys.getfilesystemencoding())
+                    yield os.fsdecode(os.path.join(self.path, line))
 
     def add_alternate_path(self, path):
         """Add an alternate path to this object store.
         """
         try:
             os.mkdir(os.path.join(self.path, INFODIR))
-        except OSError as e:
-            if e.errno != errno.EEXIST:
-                raise
+        except FileExistsError:
+            pass
         alternates_path = os.path.join(self.path, INFODIR, "alternates")
         with GitFile(alternates_path, 'wb') as f:
             try:
                 orig_f = open(alternates_path, 'rb')
-            except (OSError, IOError) as e:
-                if e.errno != errno.ENOENT:
-                    raise
+            except FileNotFoundError:
+                pass
             else:
                 with orig_f:
                     f.write(orig_f.read())
-            f.write(path.encode(sys.getfilesystemencoding()) + b"\n")
+            f.write(os.fsencode(path) + b"\n")
 
         if not os.path.isabs(path):
             path = os.path.join(self.path, path)
@@ -564,11 +611,9 @@ class DiskObjectStore(PackBasedObjectStore):
         """Read and iterate over new pack files and cache them."""
         try:
             pack_dir_contents = os.listdir(self.pack_dir)
-        except OSError as e:
-            if e.errno == errno.ENOENT:
-                self.close()
-                return []
-            raise
+        except FileNotFoundError:
+            self.close()
+            return []
         pack_files = set()
         for name in pack_dir_contents:
             if name.startswith("pack-") and name.endswith(".pack"):
@@ -600,16 +645,17 @@ class DiskObjectStore(PackBasedObjectStore):
             if len(base) != 2:
                 continue
             for rest in os.listdir(os.path.join(self.path, base)):
-                yield (base+rest).encode(sys.getfilesystemencoding())
+                sha = os.fsencode(base+rest)
+                if not valid_hexsha(sha):
+                    continue
+                yield sha
 
     def _get_loose_object(self, sha):
         path = self._get_shafile_path(sha)
         try:
             return ShaFile.from_path(path)
-        except (OSError, IOError) as e:
-            if e.errno == errno.ENOENT:
-                return None
-            raise
+        except FileNotFoundError:
+            return None
 
     def _remove_loose_object(self, sha):
         os.remove(self._get_shafile_path(sha))
@@ -632,13 +678,14 @@ class DiskObjectStore(PackBasedObjectStore):
     def _complete_thin_pack(self, f, path, copier, indexer):
         """Move a specific file containing a pack into the pack directory.
 
-        :note: The file should be on the same file system as the
+        Note: The file should be on the same file system as the
             packs directory.
 
-        :param f: Open file object for the pack.
-        :param path: Path to the pack file.
-        :param copier: A PackStreamCopier to use for writing pack data.
-        :param indexer: A PackIndexer for indexing the pack.
+        Args:
+          f: Open file object for the pack.
+          path: Path to the pack file.
+          copier: A PackStreamCopier to use for writing pack data.
+          indexer: A PackIndexer for indexing the pack.
         """
         entries = list(indexer)
 
@@ -660,7 +707,9 @@ class DiskObjectStore(PackBasedObjectStore):
             assert len(ext_sha) == 20
             type_num, data = self.get_raw(ext_sha)
             offset = f.tell()
-            crc32 = write_pack_object(f, type_num, data, sha=new_sha)
+            crc32 = write_pack_object(
+                f, type_num, data, sha=new_sha,
+                compression_level=self.pack_compression_level)
             entries.append((ext_sha, offset, crc32))
         pack_sha = new_sha.digest()
         f.write(pack_sha)
@@ -675,9 +724,8 @@ class DiskObjectStore(PackBasedObjectStore):
             # removal, silently passing if the target does not exist.
             try:
                 os.remove(target_pack)
-            except (IOError, OSError) as e:
-                if e.errno != errno.ENOENT:
-                    raise
+            except FileNotFoundError:
+                pass
         os.rename(path, target_pack)
 
         # Write the index.
@@ -701,11 +749,12 @@ class DiskObjectStore(PackBasedObjectStore):
         outside the pack. They should never be placed in the object store
         directly, and always indexed and completed as they are copied.
 
-        :param read_all: Read function that blocks until the number of
+        Args:
+          read_all: Read function that blocks until the number of
             requested bytes are read.
-        :param read_some: Read function that returns at least one byte, but may
+          read_some: Read function that returns at least one byte, but may
             not return the number of bytes requested.
-        :return: A Pack object pointing at the now-completed thin pack in the
+        Returns: A Pack object pointing at the now-completed thin pack in the
             objects/pack directory.
         """
         fd, path = tempfile.mkstemp(dir=self.path, prefix='tmp_pack_')
@@ -719,10 +768,11 @@ class DiskObjectStore(PackBasedObjectStore):
     def move_in_pack(self, path):
         """Move a specific file containing a pack into the pack directory.
 
-        :note: The file should be on the same file system as the
+        Note: The file should be on the same file system as the
             packs directory.
 
-        :param path: Path to the pack file.
+        Args:
+          path: Path to the pack file.
         """
         with PackData(path) as p:
             entries = p.sorted_entries()
@@ -740,9 +790,8 @@ class DiskObjectStore(PackBasedObjectStore):
             # removal, silently passing if the target does not exist.
             try:
                 os.remove(target_pack)
-            except (IOError, OSError) as e:
-                if e.errno != errno.ENOENT:
-                    raise
+            except FileNotFoundError:
+                pass
         os.rename(path, target_pack)
         final_pack = Pack(basename)
         self._add_cached_pack(basename, final_pack)
@@ -751,7 +800,7 @@ class DiskObjectStore(PackBasedObjectStore):
     def add_pack(self):
         """Add a new pack to this object store.
 
-        :return: Fileobject to write to, a commit function to
+        Returns: Fileobject to write to, a commit function to
             call when the pack is finished and an abort
             function.
         """
@@ -776,27 +825,27 @@ class DiskObjectStore(PackBasedObjectStore):
     def add_object(self, obj):
         """Add a single object to this object store.
 
-        :param obj: Object to add
+        Args:
+          obj: Object to add
         """
         path = self._get_shafile_path(obj.id)
         dir = os.path.dirname(path)
         try:
             os.mkdir(dir)
-        except OSError as e:
-            if e.errno != errno.EEXIST:
-                raise
+        except FileExistsError:
+            pass
         if os.path.exists(path):
             return  # Already there, no need to write again
         with GitFile(path, 'wb') as f:
-            f.write(obj.as_legacy_object())
+            f.write(obj.as_legacy_object(
+                compression_level=self.loose_compression_level))
 
     @classmethod
     def init(cls, path):
         try:
             os.mkdir(path)
-        except OSError as e:
-            if e.errno != errno.EEXIST:
-                raise
+        except FileExistsError:
+            pass
         os.mkdir(os.path.join(path, "info"))
         os.mkdir(os.path.join(path, PACKDIR))
         return cls(path)
@@ -808,6 +857,7 @@ class MemoryObjectStore(BaseObjectStore):
     def __init__(self):
         super(MemoryObjectStore, self).__init__()
         self._data = {}
+        self.pack_compression_level = -1
 
     def _to_hexsha(self, sha):
         if len(sha) == 40:
@@ -837,8 +887,9 @@ class MemoryObjectStore(BaseObjectStore):
     def get_raw(self, name):
         """Obtain the raw text for an object.
 
-        :param name: sha for the object.
-        :return: tuple with numeric type and object contents.
+        Args:
+          name: sha for the object.
+        Returns: tuple with numeric type and object contents.
         """
         obj = self[self._to_hexsha(name)]
         return obj.type_num, obj.as_raw_string()
@@ -859,7 +910,8 @@ class MemoryObjectStore(BaseObjectStore):
     def add_objects(self, objects, progress=None):
         """Add a set of objects to this object store.
 
-        :param objects: Iterable over a list of (object, path) tuples
+        Args:
+          objects: Iterable over a list of (object, path) tuples
         """
         for obj, path in objects:
             self.add_object(obj)
@@ -870,7 +922,7 @@ class MemoryObjectStore(BaseObjectStore):
         Because this object store doesn't support packs, we extract and add the
         individual objects.
 
-        :return: Fileobject to write to and a commit function to
+        Returns: Fileobject to write to and a commit function to
             call when the pack is finished.
         """
         f = BytesIO()
@@ -888,8 +940,9 @@ class MemoryObjectStore(BaseObjectStore):
     def _complete_thin_pack(self, f, indexer):
         """Complete a thin pack by adding external references.
 
-        :param f: Open file object for the pack.
-        :param indexer: A PackIndexer for indexing the pack.
+        Args:
+          f: Open file object for the pack.
+          indexer: A PackIndexer for indexing the pack.
         """
         entries = list(indexer)
 
@@ -904,7 +957,8 @@ class MemoryObjectStore(BaseObjectStore):
         for ext_sha in indexer.ext_refs():
             assert len(ext_sha) == 20
             type_num, data = self.get_raw(ext_sha)
-            write_pack_object(f, type_num, data, sha=new_sha)
+            write_pack_object(
+                f, type_num, data, sha=new_sha)
         pack_sha = new_sha.digest()
         f.write(pack_sha)
 
@@ -915,9 +969,10 @@ class MemoryObjectStore(BaseObjectStore):
         outside the pack. Because this object store doesn't support packs, we
         extract and add the individual objects.
 
-        :param read_all: Read function that blocks until the number of
+        Args:
+          read_all: Read function that blocks until the number of
             requested bytes are read.
-        :param read_some: Read function that returns at least one byte, but may
+          read_some: Read function that returns at least one byte, but may
             not return the number of bytes requested.
         """
         f, commit, abort = self.add_pack()
@@ -947,8 +1002,9 @@ class ObjectStoreIterator(ObjectIterator):
     def __init__(self, store, sha_iter):
         """Create a new ObjectIterator.
 
-        :param store: Object store to retrieve from
-        :param sha_iter: Iterator over (sha, path) tuples
+        Args:
+          store: Object store to retrieve from
+          sha_iter: Iterator over (sha, path) tuples
         """
         self.store = store
         self.sha_iter = sha_iter
@@ -975,11 +1031,12 @@ class ObjectStoreIterator(ObjectIterator):
     def __contains__(self, needle):
         """Check if an object is present.
 
-        :note: This checks if the object is present in
+        Note: This checks if the object is present in
             the underlying object store, not if it would
             be yielded by the iterator.
 
-        :param needle: SHA1 of the object to check for
+        Args:
+          needle: SHA1 of the object to check for
         """
         if needle == ZERO_SHA:
             return False
@@ -988,7 +1045,7 @@ class ObjectStoreIterator(ObjectIterator):
     def __getitem__(self, key):
         """Find an object by SHA1.
 
-        :note: This retrieves the object from the underlying
+        Note: This retrieves the object from the underlying
             object store. It will also succeed if the object would
             not be returned by the iterator.
         """
@@ -1020,10 +1077,11 @@ class ObjectStoreIterator(ObjectIterator):
 def tree_lookup_path(lookup_obj, root_sha, path):
     """Look up an object in a Git tree.
 
-    :param lookup_obj: Callback for retrieving object by SHA1
-    :param root_sha: SHA1 of the root tree
-    :param path: Path to lookup
-    :return: A tuple of (mode, SHA) of the resulting path.
+    Args:
+      lookup_obj: Callback for retrieving object by SHA1
+      root_sha: SHA1 of the root tree
+      path: Path to lookup
+    Returns: A tuple of (mode, SHA) of the resulting path.
     """
     tree = lookup_obj(root_sha)
     if not isinstance(tree, Tree):
@@ -1034,9 +1092,10 @@ def tree_lookup_path(lookup_obj, root_sha, path):
 def _collect_filetree_revs(obj_store, tree_sha, kset):
     """Collect SHA1s of files and directories for specified tree.
 
-    :param obj_store: Object store to get objects by SHA from
-    :param tree_sha: tree reference to walk
-    :param kset: set to fill with references to files and directories
+    Args:
+      obj_store: Object store to get objects by SHA from
+      tree_sha: tree reference to walk
+      kset: set to fill with references to files and directories
     """
     filetree = obj_store[tree_sha]
     for name, mode, sha in filetree.iteritems():
@@ -1054,11 +1113,12 @@ def _split_commits_and_tags(obj_store, lst, ignore_unknown=False):
     through, and unless ignore_unknown argument is True, KeyError
     is thrown for SHA1 missing in the repository
 
-    :param obj_store: Object store to get objects by SHA1 from
-    :param lst: Collection of commit and tag SHAs
-    :param ignore_unknown: True to skip SHA1 missing in the repository
+    Args:
+      obj_store: Object store to get objects by SHA1 from
+      lst: Collection of commit and tag SHAs
+      ignore_unknown: True to skip SHA1 missing in the repository
         silently.
-    :return: A tuple of (commits, tags, others) SHA1s
+    Returns: A tuple of (commits, tags, others) SHA1s
     """
     commits = set()
     tags = set()
@@ -1088,20 +1148,23 @@ def _split_commits_and_tags(obj_store, lst, ignore_unknown=False):
 class MissingObjectFinder(object):
     """Find the objects missing from another object store.
 
-    :param object_store: Object store containing at least all objects to be
+    Args:
+      object_store: Object store containing at least all objects to be
         sent
-    :param haves: SHA1s of commits not to send (already present in target)
-    :param wants: SHA1s of commits to send
-    :param progress: Optional function to report progress to.
-    :param get_tagged: Function that returns a dict of pointed-to sha -> tag
+      haves: SHA1s of commits not to send (already present in target)
+      wants: SHA1s of commits to send
+      progress: Optional function to report progress to.
+      get_tagged: Function that returns a dict of pointed-to sha -> tag
         sha for including tags.
-    :param get_parents: Optional function for getting the parents of a commit.
-    :param tagged: dict of pointed-to sha -> tag sha for including tags
+      get_parents: Optional function for getting the parents of a commit.
+      tagged: dict of pointed-to sha -> tag sha for including tags
     """
 
-    def __init__(self, object_store, haves, wants, progress=None,
+    def __init__(self, object_store, haves, wants, shallow=None, progress=None,
                  get_tagged=None, get_parents=lambda commit: commit.parents):
         self.object_store = object_store
+        if shallow is None:
+            shallow = set()
         self._get_parents = get_parents
         # process Commits and Tags differently
         # Note, while haves may list commits/tags not available locally,
@@ -1115,12 +1178,13 @@ class MissingObjectFinder(object):
         # all_ancestors is a set of commits that shall not be sent
         # (complete repository up to 'haves')
         all_ancestors = object_store._collect_ancestors(
-            have_commits, get_parents=self._get_parents)[0]
+            have_commits, shallow=shallow, get_parents=self._get_parents)[0]
         # all_missing - complete set of commits between haves and wants
         # common - commits from all_ancestors we hit into while
         # traversing parent hierarchy of wants
         missing_commits, common_commits = object_store._collect_ancestors(
-            want_commits, all_ancestors, get_parents=self._get_parents)
+            want_commits, all_ancestors, shallow=shallow,
+            get_parents=self._get_parents)
         self.sha_done = set()
         # Now, fill sha_done with commits and revisions of
         # files and directories known to be both locally
@@ -1190,8 +1254,9 @@ class ObjectStoreGraphWalker(object):
     def __init__(self, local_heads, get_parents, shallow=None):
         """Create a new instance.
 
-        :param local_heads: Heads to start search with
-        :param get_parents: Function for finding the parents of a SHA1.
+        Args:
+          local_heads: Heads to start search with
+          get_parents: Function for finding the parents of a SHA1.
         """
         self.heads = set(local_heads)
         self.get_parents = get_parents
@@ -1254,11 +1319,12 @@ def commit_tree_changes(object_store, tree, changes):
     number of changes to a big tree. For a large number of changes
     to a large tree, use e.g. commit_tree.
 
-    :param object_store: Object store to store new objects in
+    Args:
+      object_store: Object store to store new objects in
         and retrieve old ones from.
-    :param tree: Original tree root
-    :param changes: changes to apply
-    :return: New tree root object
+      tree: Original tree root
+      changes: changes to apply
+    Returns: New tree root object
     """
     # TODO(jelmer): Save up the objects and add them using .add_objects
     # rather than with individual calls to .add_object.
@@ -1349,4 +1415,4 @@ def read_packs_file(f):
         (kind, name) = line.split(b" ", 1)
         if kind != b"P":
             continue
-        yield name.decode(sys.getfilesystemencoding())
+        yield os.fsdecode(name)
